// LICENSE
// =======
//
// Copyright (c) 2009, Mohammad Abdulfatah (a.k.a. ShaderOp)
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//	* 	Redistributions of source code must retain the above copyright notice,
//		this list of conditions and the following disclaimer.
//  * 	Redistributions in binary form must reproduce the above copyright notice,
//		this list of conditions and the following disclaimer in the documentation
//		and/or other materials provided with the distribution.
//		
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//
//
// DESCRIPTION
// ===========
// Autodesk Softimage loads in a default quad-view layout, wich most users (including
// me) often change upon creating a new scene. As far as I know, these defaults can
// be changed through tinkering with some XML definition files, but I'm finding the
// process difficult to understnad. I find having to set up the views to my liking
// everytime I create a new scene to be tedious.
//
// This little plugin addresses that issue by running a small script everytime a new
// scene is created, which configures the layout based on user preferences.
//
//
// USAGE
// =====
// Install the DefaultDisplayOptionsPlugin.js file by copying it into the Softimage
// plugins directory. This will insert a menu item titled "Set Default Display
// Options..." under the "View" menu. Click on that menu item, and configure
// the settings for Views A, B, C and D in the dialog that will follow.
//
// Those settings will be automatically appied to every new scene you create,
// but will have no effect on existing scenes.
//
// CONTACT
// =======
// Please feel free to contact me if you have any comments or requests. I'm ShaderOp on
// both CGTalk and Si-Community.com.

function XSILoadPlugin( in_reg )
{
	in_reg.Author = "ShaderOp";
	in_reg.Name = "Default Display Options Plugin";
	in_reg.Email = "";
	in_reg.URL = "";
	in_reg.Major = 1;
	in_reg.Minor = 1;

	in_reg.RegisterCommand("DisplayModeSetterApplyCommand","DisplayModeSetterApplyCommand");
	in_reg.RegisterMenu(siMenuMainApplicationViewsID,"DisplayModeSetterApplyCommand_Menu",false,false);
	
	in_reg.RegisterProperty("DisplayModeSetterOptions");

	in_reg.RegisterEvent("DisplayModeSetterEndNewSceneEvent",siOnEndNewScene);
	in_reg.RegisterEvent("DisplayModeSetterStartupEvent",siOnStartup);
	in_reg.RegisterTimerEvent("DisplayModeSetterDelayedStartupEvent", 0, 1000 );
	var evTimer = Application.EventInfos( "DisplayModeSetterDelayedStartupEvent" );
	evTimer.Mute = true;

	//RegistrationInsertionPoint - do not remove this line


	return true;
}

function XSIUnloadPlugin( in_reg )
{
	return true;
}

function DisplayModeSetterApplyCommand_Init( in_ctxt )
{
	var oCmd;
	oCmd = in_ctxt.Source;
	oCmd.Description = "Sets default display mode and view options for new scenes.";
	oCmd.ReturnValue = true;

	return true;
}

function DisplayModeSetterApplyCommand_Execute(  )
{
	// Check preferences to see if they were stored
	// from previous sessions.
	if (IsSettingsStored() == false)
	{
		var rtn = SIAddProp("DisplayModeSetterOptions", "Scene_Root");
		var propertySetFullName = "Scene_Root." + rtn(0);
		InstallCustomPreferences(propertySetFullName);
	}

	InspectObj("preferences.DisplayModeSetterOptions");
	
	return true;
}

function DisplayModeSetterApplyCommand_Menu_Init( in_ctxt )
{
	var oMenu;
	oMenu = in_ctxt.Source;
	oMenu.AddCommandItem("Set Default Display Options...","DisplayModeSetterApplyCommand");
	return true;
}



function DisplayModeSetterDelayedStartupEvent_OnEvent( in_ctxt )
{
	LogMessage("DisplayModeSetterDelayedStartupEvent_OnEvent");
	ConfigureViews();
		
	return false;
}


function DisplayModeSetterEndNewSceneEvent_OnEvent( in_ctxt )
{
	LogMessage("DisplayModeSetterEndNewSceneEvent_OnEvent");
	ConfigureViews();
		
	return false;
}

function DisplayModeSetterStartupEvent_OnEvent( in_ctxt )
{
	LogMessage("DisplayModeSetterStartupEvent_OnEvent");
	var evTimer = Application.EventInfos( "DisplayModeSetterDelayedStartupEvent" );
	evTimer.Mute = false;
	
	return false;
}

function DisplayModeSetterOptions_Define( in_ctxt )
{
	var oCustomProperty;
	oCustomProperty = in_ctxt.Source;
	oCustomProperty.AddParameter2("viewAHeadlight",siBool,false,null,null,null,null,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewBHeadlight",siBool,false,null,null,null,null,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewCHeadlight",siBool,false,null,null,null,null,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewDHeadlight",siBool,false,null,null,null,null,siClassifUnknown,siPersistable);
	
	oCustomProperty.AddParameter2("viewAWireFrameOnShaded",siBool,false,null,null,null,null,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewBWireFrameOnShaded",siBool,false,null,null,null,null,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewCWireFrameOnShaded",siBool,false,null,null,null,null,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewDWireFrameOnShaded",siBool,false,null,null,null,null,siClassifUnknown,siPersistable);
	
	oCustomProperty.AddParameter2("viewADisplayMode",siInt4,0,0,10,0,10,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewBDisplayMode",siInt4,0,0,10,0,10,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewCDisplayMode",siInt4,0,0,10,0,10,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewDDisplayMode",siInt4,0,0,10,0,10,siClassifUnknown,siPersistable);
	
	oCustomProperty.AddParameter2("viewACameraView",siInt4,0,0,10,0,10,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewBCameraView",siInt4,0,0,10,0,10,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewCCameraView",siInt4,0,0,10,0,10,siClassifUnknown,siPersistable);
	oCustomProperty.AddParameter2("viewDCameraView",siInt4,0,0,10,0,10,siClassifUnknown,siPersistable);

	
	return true;
}

function DisplayModeSetterOptions_DefineLayout( in_ctxt )
{
	/*
	Display mode keys:
	   "wireframe" = 0 
	   "hiddenline" = 1
	   "shaded" = 2
	   "textured" = 3
	*/

	/*
	Camera view keys
		"default" = 0;
		"top"	= 1
		"front"	= 2
		"right"	= 3
		"user"	= 4
	*/
	var displayModes = new Array( 
	   "Wireframe", 0, 
	   "Hidden line removal",  1,
	   "Shaded", 2,
	   "Textured", 3
	);

	var cameraViews = new Array( 
	   "Default", 0, 
	   "Top",  1,
	   "Front",  2,
	   "Right", 3,
	   "User", 4
	);


	var oLayout,oItem;
	oLayout = in_ctxt.Source;
	oLayout.Clear();

	oLayout.AddTab("View A");
	
	oLayout.AddGroup("", true);
	oLayout.AddEnumControl("viewACameraView", cameraViews, "Camera", siControlRadio);			
	oLayout.EndGroup();

	oLayout.AddGroup("", true);
	oLayout.AddEnumControl("viewADisplayMode", displayModes, "Display mode", siControlRadio);
	oLayout.EndGroup();

	oLayout.AddGroup("", true);
	oLayout.AddItem("viewAHeadlight", "Headlight");	
	oLayout.AddItem("viewAWireFrameOnShaded", "Wireframe on shaded");
	oLayout.EndGroup();


	oLayout.AddTab("View B");
	
	oLayout.AddGroup("", true);
	oLayout.AddEnumControl("viewBCameraView", cameraViews, "Camera", siControlRadio);			
	oLayout.EndGroup();

	oLayout.AddGroup("", true);
	oLayout.AddEnumControl("viewBDisplayMode", displayModes, "Display mode", siControlRadio);
	oLayout.EndGroup();

	oLayout.AddGroup("", true);
	oLayout.AddItem("viewBHeadlight", "Headlight");	
	oLayout.AddItem("viewBWireFrameOnShaded", "Wireframe on shaded");
	oLayout.EndGroup();


	oLayout.AddTab("View C");
	
	oLayout.AddGroup("", true);
	oLayout.AddEnumControl("viewCCameraView", cameraViews, "Camera", siControlRadio);			
	oLayout.EndGroup();

	oLayout.AddGroup("", true);
	oLayout.AddEnumControl("viewCDisplayMode", displayModes, "Display mode", siControlRadio);
	oLayout.EndGroup();

	oLayout.AddGroup("", true);
	oLayout.AddItem("viewCHeadlight", "Headlight");	
	oLayout.AddItem("viewCWireFrameOnShaded", "Wireframe on shaded");
	oLayout.EndGroup();


	oLayout.AddTab("View D");
	
	oLayout.AddGroup("", true);
	oLayout.AddEnumControl("viewDCameraView", cameraViews, "Camera", siControlRadio);			
	oLayout.EndGroup();

	oLayout.AddGroup("", true);
	oLayout.AddEnumControl("viewDDisplayMode", displayModes, "Display mode", siControlRadio);
	oLayout.EndGroup();

	oLayout.AddGroup("", true);
	oLayout.AddItem("viewDHeadlight", "Headlight");	
	oLayout.AddItem("viewDWireFrameOnShaded", "Wireframe on shaded");
	oLayout.EndGroup();


	return true;
}

function IsSettingsStored()
{
	var prefs = Application.Preferences;
	var categoryCount = prefs.Categories.Count;

	for (var i = 0; i < categoryCount; ++i)
	{
		var categoryName = prefs.Categories(i).Name;
		if (categoryName == "DisplayModeSetterOptions")
		{
			return true;
		}
	}
	
	return false;
}

function ConfigureViews()
{
	if (IsSettingsStored() == false)
		return true;
		
	// First, set up the view camera for all views...
	if (GetValue("preferences.DisplayModeSetterOptions.viewACameraView") == 1)
		SetViewCamera("top", 0);		
	else if (GetValue("preferences.DisplayModeSetterOptions.viewACameraView") == 2)
		SetViewCamera("front", 0);
	else if (GetValue("preferences.DisplayModeSetterOptions.viewACameraView") == 3)
		SetViewCamera("right", 0);
	else if (GetValue("preferences.DisplayModeSetterOptions.viewACameraView") == 4)
		SetViewCamera("user", 0);

	if (GetValue("preferences.DisplayModeSetterOptions.viewBCameraView") == 1)
		SetViewCamera("top", 1);		
	else if (GetValue("preferences.DisplayModeSetterOptions.viewBCameraView") == 2)
		SetViewCamera("front", 1);
	else if (GetValue("preferences.DisplayModeSetterOptions.viewBCameraView") == 3)
		SetViewCamera("right", 1);
	else if (GetValue("preferences.DisplayModeSetterOptions.viewBCameraView") == 4)
		SetViewCamera("user", 1);

	if (GetValue("preferences.DisplayModeSetterOptions.viewCCameraView") == 1)
		SetViewCamera("top", 2);		
	else if (GetValue("preferences.DisplayModeSetterOptions.viewCCameraView") == 2)
		SetViewCamera("front", 2);
	else if (GetValue("preferences.DisplayModeSetterOptions.viewCCameraView") == 3)
		SetViewCamera("right", 2);
	else if (GetValue("preferences.DisplayModeSetterOptions.viewCCameraView") == 4)
		SetViewCamera("user", 2);

	if (GetValue("preferences.DisplayModeSetterOptions.viewDCameraView") == 1)
		SetViewCamera("top", 3);		
	else if (GetValue("preferences.DisplayModeSetterOptions.viewDCameraView") == 2)
		SetViewCamera("front", 3);
	else if (GetValue("preferences.DisplayModeSetterOptions.viewDCameraView") == 3)
		SetViewCamera("right", 3);
	else if (GetValue("preferences.DisplayModeSetterOptions.viewDCameraView") == 4)
		SetViewCamera("user", 3);


	// Now the display mode...
	var cameraName = "";
	
	cameraName = GetViewCamera(0);
	LogMessage("CameraName = " + cameraName);
	if (GetValue("preferences.DisplayModeSetterOptions.viewADisplayMode") == 0)
		SetDisplayMode(cameraName, "wireframe");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewADisplayMode") == 1)
		SetDisplayMode(cameraName, "hiddenline");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewADisplayMode") == 2)
		SetDisplayMode(cameraName, "shaded");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewADisplayMode") == 3)
		SetDisplayMode(cameraName, "textured");
	
	if (GetValue("preferences.DisplayModeSetterOptions.viewAHeadlight") == true)
		SetValue(cameraName + ".camdisp.headlight", true, null);
		
	if (GetValue("preferences.DisplayModeSetterOptions.viewAWireFrameOnShaded") == true)
		SetValue(cameraName + ".camdisp.wireontopunsel", true, null);


	cameraName = GetViewCamera(1);
	if (GetValue("preferences.DisplayModeSetterOptions.viewBDisplayMode") == 0)
		SetDisplayMode(cameraName, "wireframe");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewBDisplayMode") == 1)
		SetDisplayMode(cameraName, "hiddenline");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewBDisplayMode") == 2)
		SetDisplayMode(cameraName, "shaded");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewBDisplayMode") == 3)
		SetDisplayMode(cameraName, "textured");

	if (GetValue("preferences.DisplayModeSetterOptions.viewBHeadlight") == true)
		SetValue(cameraName + ".camdisp.headlight", true, null);
		
	if (GetValue("preferences.DisplayModeSetterOptions.viewBWireFrameOnShaded") == true)
		SetValue(cameraName + ".camdisp.wireontopunsel", true, null);

	
	cameraName = GetViewCamera(2);
	if (GetValue("preferences.DisplayModeSetterOptions.viewCDisplayMode") == 0)
		SetDisplayMode(cameraName, "wireframe");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewCDisplayMode") == 1)
		SetDisplayMode(cameraName, "hiddenline");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewCDisplayMode") == 2)
		SetDisplayMode(cameraName, "shaded");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewCDisplayMode") == 3)
		SetDisplayMode(cameraName, "textured");
		
	if (GetValue("preferences.DisplayModeSetterOptions.viewCHeadlight") == true)
		SetValue(cameraName + ".camdisp.headlight", true, null);
		
	if (GetValue("preferences.DisplayModeSetterOptions.viewCWireFrameOnShaded") == true)
		SetValue(cameraName + ".camdisp.wireontopunsel", true, null);
		

	cameraName = GetViewCamera(3);
	if (GetValue("preferences.DisplayModeSetterOptions.viewDDisplayMode") == 0)
		SetDisplayMode(cameraName, "wireframe");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewDDisplayMode") == 1)
		SetDisplayMode(cameraName, "hiddenline");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewDDisplayMode") == 2)
		SetDisplayMode(cameraName, "shaded");
	else if (GetValue("preferences.DisplayModeSetterOptions.viewDDisplayMode") == 3)
		SetDisplayMode(cameraName, "textured");

	if (GetValue("preferences.DisplayModeSetterOptions.viewDHeadlight") == true)
		SetValue(cameraName + ".camdisp.headlight", true, null);
		
	if (GetValue("preferences.DisplayModeSetterOptions.viewDWireFrameOnShaded") == true)
		SetValue(cameraName + ".camdisp.wireontopunsel", true, null);

}